import Flutter
import UIKit
import appmate

public class SwiftAppmateFlutterPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "appmate_flutter", binaryMessenger: registrar.messenger())
        let instance = SwiftAppmateFlutterPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "setApiKey" {
            setApiKey(call, result: result)
        } else if call.method == "setSandboxActive" {
            setSandboxActive(call, result: result)
        } else if call.method == "setUserId" {
            setUserId(call, result: result)
        } else if call.method == "getUserId" {
            getUserId(call, result: result)
        } else if call.method == "getProducts" {
            getProducts(call, result: result)
        } else if call.method == "setOneSignalId" {
            setOneSignalId(call, result: result)
        } else if call.method == "getProductsWithType" {
            getProductsWithType(call, result: result)
        } else if call.method == "getProductsWithIdList" {
            getProductsWithIdList(call, result: result)
        } else if call.method == "getCurrentPurchases" {
            getCurrentPurchases(call, result: result)
        } else if call.method == "getAllPurchases" {
            getAllPurchases(call, result: result)
        } else if call.method == "makePurchase" {
            makePurchase(call, result: result)
        } else if call.method == "restorePurchases" {
            restorePurchases(call, result: result)
        } else if call.method == "consumePurchase" {
            consumePurchase(call, result: result)
        } else if call.method == "consumePurchases" {
            consumePurchases(call, result: result)
        } else if call.method == "unsubscribe" {
            unsubscribe(call, result: result)
        } else if call.method == "createUserIdRelation" {
            createUserIdRelation(call, result: result)
        } else if call.method == "deleteMasterUserIdRelation" {
            deleteMasterUserIdRelation(call, result: result)
        } else if call.method == "deleteSubUserIdRelation" {
            deleteSubUserIdRelation(call, result: result)
        } else if call.method == "setRuntimeEnvironment" {
            setRuntimeEnvironment(call, result: result)
        } else if call.method == "isProductPurchased" {
            isProductPurchased(call, result: result)
        } else if call.method == "getEntitlements" {
            getEntitlements(call, result: result);
        } else if call.method == "setAppmateEvent" {
            setAppmateEvent(call, result: result)
        } else if call.method == "setAdvertiseId" {
            setAdvertiseId(call, result: result)
        } else if call.method == "setAppsFlyerId" {
            setAppsFlyerId(call, result: result)
        } else if call.method == "setUserAttributes" {
            setUserAttributes(call, result: result)
        } else if call.method == "getOfferwall" {
            getOfferwall(call, result: result)
        } else if call.method == "enableDebugLogs" {
            enableDebugLogs(call)
        }
    }
    
    private func setApiKey(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let apiKey = args["apiKey"] as? String {
            PurchaseClient.shared.setApiKey(apiKey)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func setSandboxActive(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let isActive = args["isActive"] as? Bool {
            PurchaseClient.shared.setSandboxActive(isActive)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func setUserId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let userId = args["userId"] as? String {
            PurchaseClient.shared.setUserId(userId)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getUserId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        result(PurchaseClient.shared.getUserId())
    }

    // TODO: düzenlenecek test edilecek
    private func setOneSignalId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let oneSignalId = args["oneSignalId"] as? String {
            PurchaseClient.shared.setOneSignalId(oneSignalId)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getProducts(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getProducts { products, error in
            if let products = products {
                result([products.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func getProductsWithType(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let type = args["type"] as? String {
            PurchaseClient.shared.getProductsWithType(type) { products, error in
                if let products = products {
                    result([products.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getProductsWithIdList(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let ids = args["ids"] as? [String] {
            PurchaseClient.shared.getProductsWithIdList(ids) { products, error in
                if let products = products {
                    result([products.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getCurrentPurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getCurrentPurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func getAllPurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getAllPurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func makePurchase(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        let decoder = DescriptionDecoder()
        
        if let args = call.arguments as? Dictionary<String, Any>, let strProduct = args["product"] as? String, let product = decoder.decode(Product.self, from: strProduct) {
            PurchaseClient.shared.makePurchase(with: product) { purchaseResult, purchaseCode in
                if purchaseResult != nil {
                    result([purchaseResult!.description, nil])
                } else {
                    let error = GenericError(message: purchaseCode.description)
                    result([nil, error.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func restorePurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.restorePurchases { purchases, error in
            if let purchases = purchases {
                result([purchases.description, nil])
            } else {
                result([nil, error?.description])
            }
        }
    }
    
    private func consumePurchase(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let purchaseToken = args["purchaseToken"] as? String {
            PurchaseClient.shared.consumePurchaseWithPurchaseToken(purchaseToken) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func consumePurchases(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let purchaseTokens = args["purchaseTokens"] as? [String] {
            PurchaseClient.shared.consumePurchasesWithPurchaseTokens(purchaseTokens) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func unsubscribe(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.unsubscribe()
        result([true, nil])
    }
    
    private func createUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let masterUserId = args["masterUserId"] as? String {
            PurchaseClient.shared.createUserIdRelation(with: masterUserId) { response, error in
                if let response = response, let data = response.data {
                    result([data.description, nil])
                } else {
                    result([nil, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func deleteMasterUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let masterUserId = args["masterUserId"] as? String {
            PurchaseClient.shared.deleteMasterUserIdRelation(for: masterUserId) { response, error in
                if let response = response {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func deleteSubUserIdRelation(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let subUserId = args["subUserId"] as? String {
            PurchaseClient.shared.deleteSubUserIdRelation(for: subUserId) { response, error in
                if response != nil {
                    result([true, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func setRuntimeEnvironment(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let environment = args["environment"] as? String, (environment == "test" || environment == "prod" || environment == "preprod") {
            PurchaseClient.shared.setRuntimeEnvironment(environment)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func isProductPurchased(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let productId = args["productId"] as? String {
            PurchaseClient.shared.isProductPurchased(productId: productId) { isPurchased, error in
                result([isPurchased, error?.description])
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([false, error.description])
        }
    }
    
    private func getEntitlements(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        PurchaseClient.shared.getEntitlements { entilements, error in
            if let entilements = entilements {
                result([entilements, nil])
            } else{
                result([nil, error?.description])
            }
            
        }
    }

    private func setAppmateEvent(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let productId = args["productId"] as? String, let eventType = args["userEvent"] as? String {
            var userEventType: UserEventType?
            switch eventType {
            case "1":
                userEventType = .AppLaunch
                break
            case "2":
                userEventType = .ViewProduct
                break
            case "3":
                userEventType = .PurchaseProduct
                break
            default:
                result([false, "bad arguments"])
                break
            }
            PurchaseClient.shared.saveAppUserEvents(event: .init(productId: productId, userEventType: userEventType!)) { success, error in
                if success {
                    result([success, nil])
                } else {
                    result([false, error?.description])
                }
            }
        } else {
            result([false, "bad arguments"])
        }
    }

    private func setAdvertiseId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let ADID = args["ADID"] as? String {
            PurchaseClient.shared.setIDFA(ADID)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }

    private func setAppsFlyerId(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let appsFlyerId = args["appsFlyerId"] as? String {
            PurchaseClient.shared.setAppsFlyerId(appsFlyerId)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }

    private func setUserAttributes(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let userAttributes = args["userAttributes"] as? Dictionary<String, Any> {
            PurchaseClient.shared.setUserAttributes(userAttributes)
            result([true, nil])
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func getOfferwall(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>, let offeringId = args["offeringId"] as? String {
            PurchaseClient.shared.getOfferWall(offeringId) { (offerWall, error) in
                guard let offerWall = offerWall else {
                    result([nil, error?.description])
                    return
                }
                result([offerWall.description, nil])
            }
        } else {
            let error = GenericError(message: "bad arguments")
            result([nil, error.description])
        }
    }
    
    private func enableDebugLogs(_ call: FlutterMethodCall) {
        if let args = call.arguments as? Dictionary<String, Any>, let enable = args["enable"] as? Bool {
            PurchaseClient.shared.enableDebugLogs(enable)
        }
    }
}


//TODO: - Make sure to add this to the native bridge than remove it from this file
extension OfferWall {
    override public var description: String {
        let encoder = JSONEncoder()
        encoder.outputFormatting = .prettyPrinted
        if let encoded = try? encoder.encode(self) {
            return String(data: encoded, encoding: .utf8) ?? ""
        }
        return ""
    }
}
