import 'dart:io';

import 'package:appmate_flutter/models/PriceInfo.dart';
import 'package:appmate_flutter/models/PriceTier.dart';
import 'package:appmate_flutter/models/ProductLocale.dart';
import 'package:appmate_flutter/models/ProductOffer.dart';
import 'package:appmate_flutter/models/SubscriptionDuration.dart';
import 'package:appmate_flutter/models/SyncInfo.dart';

/// In-app purchase product such as consumables, non-consumables or subscriptions.
///
/// In order to use in-app purchases, first you must create products.
/// There are 3 different product types:
///    - Consumables
///    - Non-consumables
///    - Subscriptions
///        - Auto renewable subscriptions
///        - Non-renewable subscriptions.
class Product {
  final String? productId;
  final String? uspAppId;
  final String? defaultLanguage;
  final List<ProductLocale>? productLocales;
  final String? purchaseType;
  final String? status;
  final PriceInfo? defaultPrice;
  final PriceInfo? originalPrice;
  final List<PriceInfo>? prices;
  final List<String>? entitlements;
  final String? subsPeriod;
  final String? subGroupId;
  final String? subGroupName;
  final String? promotionId;
  final String? trialPeriod;
  final String? introductoryPeriod;
  final PriceInfo? disDefaultPrice;
  final PriceInfo? disPrice;
  final List<PriceInfo>? disPrices;
  final String? promotionalPrice;
  final String? gracePeriod;
  final String? appleSyncStatus;
  final SyncInfo? appleSyncInfo;
  final String? huaweiPromotionPriority;
  final String? googleSyncStatus;
  final SyncInfo? googleSyncInfo;
  final String? huaweiSyncStatus;
  final SyncInfo? huaweiSyncInfo;
  final List<PriceTier>? applePriceDetails;
  final PriceTier? appleActivePriceDetail;
  final SubscriptionDuration? subscriptionDuration;
  final ProductOffer? productOffer;
  final String? appleCurrency;
  final String? applePrice;

  Product(
      this.productId,
      this.uspAppId,
      this.defaultLanguage,
      this.productLocales,
      this.purchaseType,
      this.appleCurrency,
      this.applePrice,
      this.status,
      this.defaultPrice,
      this.prices,
      this.entitlements,
      this.subsPeriod,
      this.subGroupId,
      this.subGroupName,
      this.promotionId,
      this.trialPeriod,
      this.introductoryPeriod,
      this.disDefaultPrice,
      this.disPrice,
      this.disPrices,
      this.gracePeriod,
      this.appleSyncStatus,
      this.appleSyncInfo,
      this.huaweiPromotionPriority,
      this.googleSyncStatus,
      this.googleSyncInfo,
      this.huaweiSyncStatus,
      this.huaweiSyncInfo,
      this.applePriceDetails,
      this.appleActivePriceDetail,
      this.originalPrice,
      this.promotionalPrice,
      this.subscriptionDuration,
      this.productOffer);

  Product.fromJson(Map<String, dynamic> json)
      : productId = json['productId'],
        uspAppId = json['uspAppId'],
        defaultLanguage = json['defaultLanguage'],
        productLocales = json['productLocales'] != null
            ? List<ProductLocale>.from(json['productLocales']
                .map((model) => ProductLocale.fromJson(model)))
            : null,
        purchaseType = json['purchaseType'],
        appleCurrency = json['appleCurrency'],
        applePrice = json['applePrice'],
        status = json['status'],
        defaultPrice = json['defaultPrice'] != null
            ? PriceInfo.fromJson(json['defaultPrice'])
            : null,
        prices = json['prices'] != null
            ? List<PriceInfo>.from(
                json['prices'].map((model) => PriceInfo.fromJson(model)))
            : null,
        entitlements = json['entitlements'] != null
            ? List.from(json['entitlements'])
            : null,
        subsPeriod = json['subsPeriod'],
        subGroupId = json['subGroupId'],
        subGroupName = json['subGroupName'],
        promotionId = json['promotionId'],
        trialPeriod = json['trialPeriod'],
        introductoryPeriod = json['introductoryPeriod'],
        disDefaultPrice = json['disDefaultPrice'] != null
            ? PriceInfo.fromJson(json['disDefaultPrice'])
            : null,
        disPrice = json['disPrice'] != null
            ? PriceInfo.fromJson(json['disPrice'])
            : null,
        disPrices = json['disPrices'] != null
            ? List<PriceInfo>.from(
                json['disPrices'].map((model) => PriceInfo.fromJson(model)))
            : null,
        gracePeriod = json['gracePeriod'],
        appleSyncStatus = json['appleSyncStatus'],
        appleSyncInfo = json['appleSyncInfo'] != null
            ? SyncInfo.fromJson(json['appleSyncInfo'])
            : null,
        huaweiPromotionPriority = json['huaweiPromotionPriority'],
        googleSyncStatus = json['googleSyncStatus'],
        googleSyncInfo = json['googleSyncInfo'] != null
            ? SyncInfo.fromJson(json['googleSyncInfo'])
            : null,
        huaweiSyncStatus = json['huaweiSyncStatus'],
        huaweiSyncInfo = json['huaweiSyncInfo'] != null
            ? SyncInfo.fromJson(json['huaweiSyncInfo'])
            : null,
        applePriceDetails = json['applePriceDetails'] != null
            ? List<PriceTier>.from(json['applePriceDetails']
                .map((model) => PriceTier.fromJson(model)))
            : null,
        appleActivePriceDetail = json['appleActivePriceDetail'] != null
            ? PriceTier.fromJson(json['appleActivePriceDetail'])
            : null,
        promotionalPrice =
            json['promotionalPrice'] != null ? json['promotionalPrice'] : null,
        originalPrice = json['originalPrice'] != null
            ? PriceInfo.fromJson(json['originalPrice'])
            : null,
        subscriptionDuration = json['subscriptionDuration'] != null
            ? SubscriptionDuration.fromJson(json['subscriptionDuration'])
            : null,
        productOffer = json['productOffer'] != null
            ? ProductOffer.fromJson(json['productOffer'])
            : null;

  Map<String, dynamic> toJson() => {
        'productId': productId,
        'uspAppId': uspAppId,
        'defaultLanguage': defaultLanguage,
        'productLocales': productLocales != null
            ? List<dynamic>.from(productLocales!.map((model) => model.toJson()))
            : null,
        'purchaseType': purchaseType,
        'applePrice': applePrice,
        'appleCurrency': appleCurrency,
        'status': status,
        'defaultPrice': defaultPrice != null ? defaultPrice!.toJson() : null,
        'prices': prices != null
            ? List<dynamic>.from(prices!.map((model) => model.toJson()))
            : null,
        'entitlements': entitlements,
        'subsPeriod': subsPeriod,
        'subGroupId': subGroupId,
        'subGroupName': subGroupName,
        'promotionId': promotionId,
        'trialPeriod': trialPeriod,
        'introductoryPeriod': introductoryPeriod,
        'disDefaultPrice':
            disDefaultPrice != null ? disDefaultPrice!.toJson() : null,
        'disPrice': disPrice != null ? disPrice!.toJson() : null,
        'disPrices': disPrices != null
            ? List<dynamic>.from(disPrices!.map((model) => model.toJson()))
            : null,
        'gracePeriod': gracePeriod,
        'appleSyncStatus': appleSyncStatus,
        'appleSyncInfo': appleSyncInfo != null ? appleSyncInfo!.toJson() : null,
        'huaweiPromotionPriority': huaweiPromotionPriority,
        'googleSyncStatus': googleSyncStatus,
        'googleSyncInfo':
            googleSyncInfo != null ? googleSyncInfo!.toJson() : null,
        'huaweiSyncStatus': huaweiSyncStatus,
        'huaweiSyncInfo':
            huaweiSyncInfo != null ? huaweiSyncInfo!.toJson() : null,
        'applePriceDetails': applePriceDetails != null
            ? List<dynamic>.from(
                applePriceDetails!.map((model) => model.toJson()))
            : null,
        'appleActivePriceDetail': appleActivePriceDetail != null
            ? appleActivePriceDetail!.toJson()
            : null,
        'promotionalPrice': promotionalPrice,
        'originalPrice': originalPrice != null ? originalPrice!.toJson() : null,
        'subscriptionDuration': subscriptionDuration != null
            ? subscriptionDuration!.toJson()
            : null,
      };

  /// Returns the product price regardless of platform information
  String price() {
    if (Platform.isAndroid) {
      var price = defaultPrice?.price ?? "";
      if (price.length > 0) {
        return (double.parse(price) / 100).toStringAsFixed(2);
      }
    } else if (Platform.isIOS) {
      if (purchaseType == "2") {
        var subsPrice = prices?.firstWhere(
            (element) => element.currency == (appleCurrency ?? ""));
        return subsPrice?.price ?? "";
      }
      return appleActivePriceDetail?.price ?? "";
    }
    return "";
  }

  /// Returns the product currency regardless of platform information
  String currency() {
    if (Platform.isAndroid) {
      return defaultPrice?.currency ?? "";
    } else if (Platform.isIOS) {
      if (purchaseType == "2") {
        var subsPrice = prices?.firstWhere(
            (element) => element.currency == (appleCurrency ?? ""));
        return subsPrice?.currency ?? "";
      }
      return appleActivePriceDetail?.currency ?? "";
    }
    return "";
  }
}
